@@ -1,60 +1,57 @@
-{-# LANGUAGE CPP                 #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE GADTs               #-}
-{-# LANGUAGE RankNTypes          #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TypeOperators       #-}
-#if __GLASGOW_HASKELL__ >= 706
-{-# LANGUAGE PolyKinds           #-}
-#endif
-#if __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE RoleAnnotations #-}
-#endif
+{-# LANGUAGE CPP                      #-}
+{-# LANGUAGE GADTs                    #-}
+{-# LANGUAGE PolyKinds                #-}
+{-# LANGUAGE RankNTypes               #-}
+{-# LANGUAGE RoleAnnotations          #-}
+{-# LANGUAGE Safe                     #-}
+{-# LANGUAGE ScopedTypeVariables      #-}
+{-# LANGUAGE TypeOperators            #-}
 #if __GLASGOW_HASKELL__ >= 810
 {-# LANGUAGE StandaloneKindSignatures #-}
 #endif
-#if (__GLASGOW_HASKELL__ >= 704 && __GLASGOW_HASKELL__ < 707) || __GLASGOW_HASKELL__ >= 801
-{-# LANGUAGE Safe                #-}
-#elif __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy         #-}
-#endif
 module Data.GADT.Internal where
 
 import Control.Applicative  (Applicative (..))
 import Data.Functor.Product (Product (..))
 import Data.Functor.Sum     (Sum (..))
+import Data.Kind            (Type)
 import Data.Maybe           (isJust, isNothing)
 import Data.Monoid          (Monoid (..))
 import Data.Semigroup       (Semigroup (..))
-import Data.Type.Equality   ((:~:) (..))
+import Data.Type.Equality   (TestEquality (..), (:~:) (..), (:~~:) (..))
+import GHC.Generics         ((:*:) (..), (:+:) (..))
 
-#if __GLASGOW_HASKELL__ >=708
-import Data.Typeable (Typeable)
-#endif
-
-#if MIN_VERSION_base(4,10,0)
-import           Data.Type.Equality (testEquality)
-import qualified Type.Reflection    as TR
-#endif
+import qualified Type.Reflection as TR
 
 #if __GLASGOW_HASKELL__ >= 810
-import Data.Kind (Type, Constraint)
+import Data.Kind (Constraint)
 #endif
 
 -- $setup
--- >>> :set -XKindSignatures -XGADTs
+-- >>> :set -XKindSignatures -XGADTs -XTypeOperators -XStandaloneDeriving -XQuantifiedConstraints
+-- >>> import Data.Type.Equality
+-- >>> import Data.Functor.Sum
+-- >>> import Data.Maybe (isJust, isNothing)
+-- >>> import GHC.Generics
 
 -- |'Show'-like class for 1-type-parameter GADTs.  @GShow t => ...@ is equivalent to something
 -- like @(forall a. Show (t a)) => ...@.  The easiest way to create instances would probably be
 -- to write (or derive) an @instance Show (T a)@, and then simply say:
 --
--- > instance GShow t where gshowsPrec = showsPrec
+-- > instance GShow t where gshowsPrec = defaultGshowsPrec
 #if __GLASGOW_HASKELL__ >= 810
 type GShow :: (k -> Type) -> Constraint
 #endif
 class GShow t where
     gshowsPrec :: Int -> t a -> ShowS
 
+-- |If 'f' has a 'Show (f a)' instance, this function makes a suitable default
+-- implementation of 'gshowsPrec'.
+--
+-- @since 1.0.4
+defaultGshowsPrec :: Show (t a) => Int -> t a -> ShowS
+defaultGshowsPrec = showsPrec
+
 gshows :: GShow t => t a -> ShowS
 gshows = gshowsPrec (-1)
 
@@ -64,10 +61,12 @@ gshow x = gshows x ""
 instance GShow ((:~:) a) where
     gshowsPrec _ Refl = showString "Refl"
 
-#if MIN_VERSION_base(4,10,0)
+-- | @since 1.0.4
+instance GShow ((:~~:) a) where
+    gshowsPrec _ HRefl = showString "HRefl"
+
 instance GShow TR.TypeRep where
     gshowsPrec = showsPrec
-#endif
 
 --
 -- | >>> gshow (InL Refl :: Sum ((:~:) Int) ((:~:) Bool) Int)
@@ -86,6 +85,26 @@ instance (GShow a, GShow b) => GShow (Product a b) where
         . showChar ' '
         . gshowsPrec 11 y
 
+--
+-- | >>> gshow (L1 Refl :: ((:~:) Int :+: (:~:) Bool) Int)
+-- "L1 Refl"
+--
+-- @since 1.0.4
+instance (GShow a, GShow b) => GShow (a :+: b) where
+    gshowsPrec d = \s -> case s of
+        L1 x -> showParen (d > 10) (showString "L1 " . gshowsPrec 11 x)
+        R1 x -> showParen (d > 10) (showString "R1 " . gshowsPrec 11 x)
+
+-- | >>> gshow (Pair Refl Refl :: Product ((:~:) Int) ((:~:) Int) Int)
+-- "Refl :*: Refl"
+--
+-- @since 1.0.4
+instance (GShow a, GShow b) => GShow (a :*: b) where
+    gshowsPrec d (x :*: y) = showParen (d > 6)
+        $ gshowsPrec 6 x
+        . showString " :*: "
+        . gshowsPrec 6 y
+
 -- |@GReadS t@ is equivalent to @ReadS (forall b. (forall a. t a -> b) -> b)@, which is
 -- in turn equivalent to @ReadS (Exists t)@ (with @data Exists t where Exists :: t a -> Exists t@)
 #if __GLASGOW_HASKELL__ >= 810
@@ -121,6 +140,9 @@ gread s g = withSome (hd [f | (f, "") <- greads s]) g where
 -- >>> greadMaybe "InL Refl" mkSome :: Maybe (Some (Sum ((:~:) Int) ((:~:) Bool)))
 -- Just (mkSome (InL Refl))
 --
+-- >>> greadMaybe "L1 Refl" mkSome :: Maybe (Some ((:~:) Int :+: (:~:) Bool))
+-- Just (mkSome (L1 Refl))
+--
 -- >>> greadMaybe "garbage" mkSome :: Maybe (Some ((:~:) Int))
 -- Nothing
 --
@@ -130,10 +152,17 @@ greadMaybe s g = case [f | (f, "") <- greads s] of
     _       -> Nothing
 
 instance GRead ((:~:) a) where
-    greadsPrec p s = readsPrec p s >>= f
-      where
-        f :: forall x. (x :~: x, String) -> [(Some ((:~:) x), String)]
-        f (Refl, rest) = return (mkSome Refl, rest)
+    greadsPrec _ = readParen False (\s ->
+        [ (S $ \k -> k (Refl :: a :~: a), t)
+        | ("Refl", t) <- lex s
+        ])
+
+-- | @since 1.0.4
+instance k1 ~ k2 => GRead ((:~~:) (a :: k1) :: k2 -> Type) where
+    greadsPrec _ = readParen False (\s ->
+        [ (S $ \k -> k (HRefl :: a :~~: a), t)
+        | ("HRefl", t) <- lex s
+        ])
 
 instance (GRead a, GRead b) => GRead (Sum a b) where
     greadsPrec d s =
@@ -147,6 +176,19 @@ instance (GRead a, GRead b) => GRead (Sum a b) where
                     | ("InR", s2) <- lex s1
                     , (r, t) <- greadsPrec 11 s2 ]) s
 
+-- | @since 1.0.4
+instance (GRead a, GRead b) => GRead (a :+: b) where
+    greadsPrec d s =
+        readParen (d > 10)
+            (\s1 -> [ (S $ \k -> withSome r (k . L1), t)
+                    | ("L1", s2) <- lex s1
+                    , (r, t) <- greadsPrec 11 s2 ]) s
+        ++
+        readParen (d > 10)
+            (\s1 -> [ (S $ \k -> withSome r (k . R1), t)
+                    | ("R1", s2) <- lex s1
+                    , (r, t) <- greadsPrec 11 s2 ]) s
+
 -------------------------------------------------------------------------------
 -- GEq
 -------------------------------------------------------------------------------
@@ -154,6 +196,75 @@ instance (GRead a, GRead b) => GRead (Sum a b) where
 -- |A class for type-contexts which contain enough information
 -- to (at least in some cases) decide the equality of types
 -- occurring within them.
+--
+-- This class is sometimes confused with 'TestEquality' from base.
+-- 'TestEquality' only checks /type equality/.
+--
+-- Consider
+--
+-- >>> data Tag a where TagInt1 :: Tag Int; TagInt2 :: Tag Int
+--
+-- The correct @'TestEquality' Tag@ instance is
+--
+-- >>> :{
+-- instance TestEquality Tag where
+--     testEquality TagInt1 TagInt1 = Just Refl
+--     testEquality TagInt1 TagInt2 = Just Refl
+--     testEquality TagInt2 TagInt1 = Just Refl
+--     testEquality TagInt2 TagInt2 = Just Refl
+-- :}
+--
+-- While we can define
+--
+-- @
+-- instance 'GEq' Tag where
+--    'geq' = 'testEquality'
+-- @
+--
+-- this will mean we probably want to have
+--
+-- @
+-- instance 'Eq' Tag where
+--    _ '==' _ = True
+-- @
+--
+-- /Note:/ In the future version of @some@ package (to be released around GHC-9.6 / 9.8) the
+-- @forall a. Eq (f a)@ constraint will be added as a constraint to 'GEq',
+-- with a law relating 'GEq' and 'Eq':
+--
+-- @
+-- 'geq' x y = Just Refl   ⇒  x == y = True        ∀ (x :: f a) (y :: f b)
+-- x == y                ≡  isJust ('geq' x y)     ∀ (x, y :: f a)
+-- @
+--
+-- So, the more useful @'GEq' Tag@ instance would differentiate between
+-- different constructors:
+--
+-- >>> :{
+-- instance GEq Tag where
+--     geq TagInt1 TagInt1 = Just Refl
+--     geq TagInt1 TagInt2 = Nothing
+--     geq TagInt2 TagInt1 = Nothing
+--     geq TagInt2 TagInt2 = Just Refl
+-- :}
+--
+-- which is consistent with a derived 'Eq' instance for 'Tag'
+--
+-- >>> deriving instance Eq (Tag a)
+--
+-- Note that even if @a ~ b@, the @'geq' (x :: f a) (y :: f b)@ may
+-- be 'Nothing' (when value terms are inequal).
+--
+-- The consistency of 'GEq' and 'Eq' is easy to check by exhaustion:
+--
+-- >>> let checkFwdGEq :: (forall a. Eq (f a), GEq f) => f a -> f b -> Bool; checkFwdGEq x y = case geq x y of Just Refl -> x == y; Nothing -> True
+-- >>> (checkFwdGEq TagInt1 TagInt1, checkFwdGEq TagInt1 TagInt2, checkFwdGEq TagInt2 TagInt1, checkFwdGEq TagInt2 TagInt2)
+-- (True,True,True,True)
+--
+-- >>> let checkBwdGEq :: (Eq (f a), GEq f) => f a -> f a -> Bool; checkBwdGEq x y = if x == y then isJust (geq x y) else isNothing (geq x y)
+-- >>> (checkBwdGEq TagInt1 TagInt1, checkBwdGEq TagInt1 TagInt2, checkBwdGEq TagInt2 TagInt1, checkBwdGEq TagInt2 TagInt2)
+-- (True,True,True,True)
+--
 #if __GLASGOW_HASKELL__ >= 810
 type GEq :: (k -> Type) -> Constraint
 #endif
@@ -175,6 +286,15 @@ class GEq f where
     -- (Making use of the 'DSum' type from <https://hackage.haskell.org/package/dependent-sum/docs/Data-Dependent-Sum.html Data.Dependent.Sum> in both examples)
     geq :: f a -> f b -> Maybe (a :~: b)
 
+-- |If 'f' has a 'GCompare' instance, this function makes a suitable default
+-- implementation of 'geq'.
+--
+-- @since 1.0.4
+defaultGeq :: GCompare f => f a -> f b -> Maybe (a :~: b)
+defaultGeq a b = case gcompare a b of
+    GEQ -> Just Refl
+    _   -> Nothing
+
 -- |If 'f' has a 'GEq' instance, this function makes a suitable default
 -- implementation of '(==)'.
 defaultEq :: GEq f => f a -> f b -> Bool
@@ -188,6 +308,10 @@ defaultNeq x y = isNothing (geq x y)
 instance GEq ((:~:) a) where
     geq (Refl :: a :~: b) (Refl :: a :~: c) = Just (Refl :: b :~: c)
 
+-- | @since 1.0.4
+instance GEq ((:~~:) a) where
+    geq (HRefl :: a :~~: b) (HRefl :: a :~~: c) = Just (Refl :: b :~: c)
+
 instance (GEq a, GEq b) => GEq (Sum a b) where
     geq (InL x) (InL y) = geq x y
     geq (InR x) (InR y) = geq x y
@@ -199,10 +323,21 @@ instance (GEq a, GEq b) => GEq (Product a b) where
         Refl <- geq y y'
         return Refl
 
-#if MIN_VERSION_base(4,10,0)
+-- | @since 1.0.4
+instance (GEq f, GEq g) => GEq (f :+: g) where
+  geq (L1 x) (L1 y) = geq x y
+  geq (R1 x) (R1 y) = geq x y
+  geq _ _ = Nothing
+
+-- | @since 1.0.4
+instance (GEq a, GEq b) => GEq (a :*: b) where
+    geq (x :*: y) (x' :*: y') = do
+        Refl <- geq x x'
+        Refl <- geq y y'
+        return Refl
+
 instance GEq TR.TypeRep where
     geq = testEquality
-#endif
 
 -------------------------------------------------------------------------------
 -- GCompare
@@ -244,9 +379,6 @@ data GOrdering a b where
     GLT :: GOrdering a b
     GEQ :: GOrdering t t
     GGT :: GOrdering a b
-#if __GLASGOW_HASKELL__ >=708
-  deriving Typeable
-#endif
 
 -- |TODO: Think of a better name
 --
@@ -289,7 +421,10 @@ class GEq f => GCompare f where
 instance GCompare ((:~:) a) where
     gcompare Refl Refl = GEQ
 
-#if MIN_VERSION_base(4,10,0)
+-- | @since 1.0.4
+instance GCompare ((:~~:) a) where
+    gcompare HRefl HRefl = GEQ
+
 instance GCompare TR.TypeRep where
     gcompare t1 t2 =
       case testEquality t1 t2 of
@@ -301,7 +436,6 @@ instance GCompare TR.TypeRep where
             EQ -> error "impossible: 'testEquality' and 'compare' \
                         \are inconsistent for TypeRep; report this \
                         \as a GHC bug"
-#endif
 
 defaultCompare :: GCompare f => f a -> f b -> Ordering
 defaultCompare x y = weakenOrdering (gcompare x y)
@@ -321,6 +455,23 @@ instance (GCompare a, GCompare b) => GCompare (Product a b) where
             GEQ -> GEQ
             GGT -> GGT
 
+-- | @since 1.0.4
+instance (GCompare f, GCompare g) => GCompare (f :+: g) where
+    gcompare (L1 x) (L1 y) = gcompare x y
+    gcompare (L1 _) (R1 _) = GLT
+    gcompare (R1 _) (L1 _) = GGT
+    gcompare (R1 x) (R1 y) = gcompare x y
+
+-- | @since 1.0.4
+instance (GCompare a, GCompare b) => GCompare (a :*: b) where
+    gcompare (x :*: y) (x' :*: y') = case gcompare x x' of
+        GLT -> GLT
+        GGT -> GGT
+        GEQ -> case gcompare y y' of
+            GLT -> GLT
+            GEQ -> GEQ
+            GGT -> GGT
+
 -------------------------------------------------------------------------------
 -- Some
 -------------------------------------------------------------------------------
@@ -375,9 +526,7 @@ newtype Some tag = S
       withSome :: forall r. (forall a. tag a -> r) -> r
     }
 
-#if __GLASGOW_HASKELL__ >= 708
 type role Some representational
-#endif
 
 -- | Constructor.
 mkSome :: tag a -> Some tag
