@@ -1,12 +1,9 @@
 {-# LANGUAGE CPP         #-}
-#if __GLASGOW_HASKELL__ >= 704
 {-# LANGUAGE Safe        #-}
-#elif __GLASGOW_HASKELL__ >= 702
-{-# LANGUAGE Trustworthy #-}
-#endif
 module Data.GADT.Show (
     -- * Showing
     GShow  (..),
+    defaultGshowsPrec,
     gshows,
     gshow,
     -- * Reading
